/*****************************************************************************
 *
 * grail - Gesture Recognition And Instantiation Library
 *
 * Copyright (C) 2010-2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include <grail.h>
#include "grail-inserter.h"
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <malloc.h>
#include <errno.h>
#include <stdlib.h>

#ifndef GRAIL_NO_LEGACY_ABI

#define DIM_FRAMES 100
#define FRAME_RATE 100

int GRAIL_PUBLIC grail_open(struct grail *ge, int fd)
{
	struct grail_impl *x;
	struct stat fs;
	int ret;

	ret = fstat(fd, &fs);
	if (ret)
		return ret;

	x = calloc(1, sizeof(*x));
	if (!x)
		return -ENOMEM;

	if (!fs.st_rdev)
		x->fptest = fdopen(fd, "r");

	x->evemu = evemu_new(x->fptest ? "fptest" : 0);
	if (!x->evemu) {
		ret = -ENOMEM;
		goto freemem;
	}
	if (x->fptest)
		ret = evemu_read(x->evemu, x->fptest) <= 0;
	else
		ret = evemu_extract(x->evemu, fd);
	if (ret)
		goto freemem;
	if (!utouch_frame_is_supported_mtdev(x->evemu)) {
		ret = -ENODEV;
		goto freemem;
	}

	if (!x->fptest) {
		x->mtdev = mtdev_new_open(fd);
		if (!x->mtdev) {
			ret = -ENOMEM;
			goto freemem;
		}
	}
	x->fh = utouch_frame_new_engine(DIM_FRAMES, DIM_TOUCH, FRAME_RATE);
	if (!x->fh) {
		ret = -ENOMEM;
		goto freedev;
	}
	ret = utouch_frame_init_mtdev(x->fh, x->evemu);
	if (ret)
		goto freeframe;

	ret = create_grail2(x, x->fh, DIM_FRAMES, NULL,
			    GRAIL_VERSION,
			    sizeof(struct grail_control),
			    sizeof(struct grail_frame),
			    sizeof(struct grail_element));
	if (ret)
		goto freeframe;

	ge->impl = x;

	ret = gin_init(ge);
	if (ret)
		goto freegrail2;

	ret = gru_init(ge);
	if (ret)
		goto freegin;

	return 0;
 freegin:
	gin_destroy(ge);
 freegrail2:
	destroy_grail2(x);
 freeframe:
	utouch_frame_delete_engine(x->fh);
 freedev:
	if (x->mtdev)
		mtdev_close_delete(x->mtdev);
 freemem:
	evemu_delete(x->evemu);
	if (x->fptest)
		fclose(x->fptest);
	free(x);
	ge->impl = 0;
	return ret;
}

void GRAIL_PUBLIC grail_close(struct grail *ge, int fd)
{
	struct grail_impl *x = ge->impl;
	gru_destroy(ge);
	gin_destroy(ge);
	destroy_grail2(x);
	utouch_frame_delete_engine(x->fh);
	if (x->mtdev)
		mtdev_close_delete(x->mtdev);
	evemu_delete(x->evemu);
	if (x->fptest)
		fclose(x->fptest);
	free(x);
	ge->impl = 0;
}

void GRAIL_PUBLIC grail_filter_abs_events(struct grail *ge, int usage)
{
	struct grail_impl *x = ge->impl;
	x->filter_abs = usage;
}

int GRAIL_PUBLIC grail_get_contacts(const struct grail *ge,
				    struct grail_contact *touch, int max_touch)
{
	const struct gesture_inserter *gin = ge->gin;
	const struct utouch_frame *frame = ge->impl->touch;
	int i;
	if (frame->num_active < max_touch)
		max_touch = frame->num_active;
	for (i = 0; i < max_touch; i++) {
		struct grail_contact *t = &touch[i];
		const struct utouch_contact *ct = frame->active[i];
		t->id = ct->id;
		t->tool_type = ct->tool_type;
		t->pos.x = ct->x;
		t->pos.y = ct->y;
		t->touch_major = ct->touch_major;
		t->touch_minor = ct->touch_minor;
		t->width_major = ct->width_major;
		t->width_minor = ct->width_minor;
		t->angle = ct->orientation;
		t->pressure = ct->pressure;
	}
	return max_touch;
}

#endif
