#include <check.h>
#include <grail.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#define TEST_TIME_MS	20000

struct test_data {
	struct grail ge;
	int fd;
	int begin[DIM_GRAIL_TYPE];
	int update[DIM_GRAIL_TYPE];
	int end[DIM_GRAIL_TYPE];
};

static int tp_get_clients(struct grail *ge,
			  struct grail_client_info *clients, int max_clients,
			  const struct grail_coord *coords, int num_coords,
			  const grail_mask_t *types, int type_bytes)
{
	memset(&clients[0], 0, sizeof(clients[0]));
	clients[0].mask[0] = 255;
	clients[0].mask[1] = 255;
	clients[0].mask[2] = 255;
	clients[0].mask[3] = 255;
	clients[0].id.client = 345;
	clients[0].id.root = 1;
	clients[0].id.event = 2;
	clients[0].id.child = 3;
	ge = 0;
	types = 0;
	type_bytes = 0;
	max_clients = 0;
	coords = 0;
	num_coords = 0;
	return 1;
}

static void tp_event(struct grail *ge, const struct input_event *ev)
{
	ge = 0;
	ev = 0;
}

static void tp_gesture(struct grail *ge, const struct grail_event *ev)
{
	struct test_data *data = ge->priv;

	switch (ev->status) {
	case GRAIL_STATUS_BEGIN:
		data->begin[ev->type]++;
		break;
	case GRAIL_STATUS_UPDATE:
		data->update[ev->type]++;
		break;
	case GRAIL_STATUS_END:
		data->end[ev->type]++;
		break;
	}
}

static int init_test_data(struct test_data *data, const char *path)
{
	int ret;

	memset(data, 0, sizeof(*data));

	data->ge.get_clients = tp_get_clients;
	data->ge.event = tp_event;
	data->ge.gesture = tp_gesture;

	data->fd = open(path, O_RDONLY | O_NONBLOCK);
	ret = grail_open(&data->ge, data->fd);
	if(ret)
		return ret;
	return 0;
}

static void term_test_data(struct test_data *data)
{
	grail_close(&data->ge, data->fd);
	close(data->fd);
}

static void test_gesture_type(const char *name, int type,
			      int nbegin, int nupdate, int nother)
{
	struct test_data data;
	char path[1024];
	int ntype = 0;
	int i;

	sprintf(path, "io/functional/%s", name);
	fail_if(init_test_data(&data, path));

	data.ge.priv = &data;
	grail_pull(&data.ge, data.fd);

	for (i = 0; i < DIM_GRAIL_TYPE; i++) {
		if (data.begin[i] || data.update[i] || data.end[i]) {
			//fprintf(stderr, "type %d %d\n", i, data.begin[i]);
			ntype++;
		}
	}

	fail_unless(data.begin[type] == nbegin);
	fail_unless(data.update[type] >= nupdate);
	fail_unless(data.end[type] == nbegin);
	fail_unless(ntype == 1 + nother);

	term_test_data(&data);
}


START_TEST(drag_1)
{
	test_gesture_type("drag-1.evemu", GRAIL_TYPE_DRAG1, 4, 4, 0);
}
END_TEST

START_TEST(drag_2)
{
	test_gesture_type("drag-2.evemu", GRAIL_TYPE_DRAG2, 3, 3, 0);
}
END_TEST

START_TEST(drag_3)
{
	test_gesture_type("drag-3.evemu", GRAIL_TYPE_EDRAG, 2, 2, 1);
}
END_TEST

START_TEST(drag_4)
{
	test_gesture_type("drag-4.evemu", GRAIL_TYPE_MDRAG, 4, 4, 0);
}
END_TEST

START_TEST(hold_1)
{
	test_gesture_type("hold-1.evemu", GRAIL_TYPE_DRAG1, 0, 0, -1);
}
END_TEST

START_TEST(pinch_2)
{
	test_gesture_type("pinch-2.evemu", GRAIL_TYPE_PINCH2, 2, 2, 0);
}
END_TEST

START_TEST(pinch_3)
{
	test_gesture_type("pinch-3.evemu", GRAIL_TYPE_EPINCH, 2, 2, 0);
}
END_TEST

START_TEST(rotate_2)
{
	test_gesture_type("rotate-2.evemu", GRAIL_TYPE_ROTATE2, 4, 4, 0);
}
END_TEST

START_TEST(tap_1)
{
	test_gesture_type("tap-1.evemu", GRAIL_TYPE_TAP1, 0, 1, 0);
}
END_TEST

START_TEST(tap_4)
{
	test_gesture_type("tap-4.evemu", GRAIL_TYPE_TAP4, 0, 1, 0);
}
END_TEST

#define ADD_TEST(suite, name)					\
	do {							\
		TCase *test = tcase_create(#name);		\
		tcase_set_timeout (test, TEST_TIME_MS / 1000);	\
		tcase_add_test(test, name);			\
		suite_add_tcase(suite, test);			\
	} while (0)

Suite *make_gesture_suite()
{
	Suite *suite = suite_create("grail-gestures");

	ADD_TEST(suite, drag_1);
	ADD_TEST(suite, drag_2);
	ADD_TEST(suite, drag_3);
	ADD_TEST(suite, drag_4);
	ADD_TEST(suite, hold_1);
	ADD_TEST(suite, pinch_2);
	ADD_TEST(suite, pinch_3);
	ADD_TEST(suite, rotate_2);
	ADD_TEST(suite, tap_1);
	ADD_TEST(suite, tap_4);

	return suite;
}
